"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _createHash = _interopRequireDefault(require("./createHash"));
var _makeRequest = _interopRequireDefault(require("./makeRequest"));
const _excluded = ["viewport", "chunks", "maxHeight"];
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }
function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }
const POLL_INTERVAL = 5000; // 5 secs
const VIEWPORT_PATTERN = /^([0-9]+)x([0-9]+)$/;
async function waitFor({
  requestId,
  endpoint,
  apiKey,
  apiSecret
}) {
  const {
    status,
    result
  } = await (0, _makeRequest.default)({
    url: `${endpoint}/api/snap-requests/${requestId}`,
    method: 'GET',
    json: true
  }, {
    apiKey,
    apiSecret,
    retryCount: 5
  });
  if (status === 'done') {
    return result.map(i => Object.assign({}, i, {
      snapRequestId: requestId
    }));
  }
  await new Promise(r => setTimeout(r, POLL_INTERVAL));
  return waitFor({
    requestId,
    endpoint,
    apiKey,
    apiSecret
  });
}
const MIN_INTERNET_EXPLORER_WIDTH = 400;
function getPageSlices(pages, chunks) {
  const extendsPages = {};
  const rawPages = [];
  for (const page of pages) {
    if (page.extends) {
      extendsPages[page.extends] = extendsPages[page.extends] || [];
      extendsPages[page.extends].push(page);
    } else {
      rawPages.push(page);
    }
  }
  const result = [];
  const pagesPerChunk = Math.ceil(rawPages.length / chunks);
  for (let i = 0; i < chunks; i += 1) {
    const pageSlice = rawPages.slice(i * pagesPerChunk, i * pagesPerChunk + pagesPerChunk);
    if (pageSlice.length > 0) {
      result.push(pageSlice);
    }
  }
  for (const sha of Object.keys(extendsPages)) {
    extendsPages[sha].extendsSha = sha;
    result.push(extendsPages[sha]);
  }
  return result;
}
class RemoteBrowserTarget {
  constructor(browserName, _ref) {
    let {
        viewport,
        chunks = 1,
        maxHeight
      } = _ref,
      otherOptions = _objectWithoutProperties(_ref, _excluded);
    const viewportMatch = viewport.match(VIEWPORT_PATTERN);
    if (!viewportMatch) {
      throw new Error(`Invalid viewport "${viewport}". Here's an example of a valid one: "1024x768".`);
    }
    const [_, width] = viewportMatch; // eslint-disable-line no-unused-vars

    if (browserName === 'edge' && width < MIN_INTERNET_EXPLORER_WIDTH) {
      throw new Error(`Invalid viewport width for the "edge" target (you provided ${width}). Smallest width it can handle is ${MIN_INTERNET_EXPLORER_WIDTH}.`);
    }
    this.chunks = chunks;
    this.browserName = browserName;
    this.viewport = viewport;
    this.maxHeight = maxHeight;
    this.otherOptions = otherOptions;
  }
  async execute({
    globalCSS,
    assetsPackage,
    staticPackage,
    snapPayloads,
    apiKey,
    apiSecret,
    endpoint,
    pages,
    asyncResults = false,
    targetName
  }) {
    const boundMakeRequest = async ({
      slice,
      chunk,
      pageSlice
    }) => {
      const payloadString = JSON.stringify(_objectSpread(_objectSpread({
        viewport: this.viewport,
        maxHeight: this.maxHeight
      }, this.otherOptions), {}, {
        globalCSS,
        snapPayloads: slice,
        chunk,
        staticPackage,
        assetsPackage,
        pages: pageSlice,
        extendsSha: pageSlice ? pageSlice.extendsSha : undefined
      }));
      const payloadHash = (0, _createHash.default)(payloadString + (pageSlice ? Math.random() : ''));
      const formData = {
        type: pageSlice && pageSlice.extendsSha ? 'extends-report' : `browser-${this.browserName}`,
        targetName,
        payloadHash,
        payload: {
          options: {
            filename: 'payload.json',
            contentType: 'application/json'
          },
          value: payloadString
        }
      };
      if (pageSlice && pageSlice.extendsSha) {
        formData.extendsSha = pageSlice.extendsSha;
      }
      return (0, _makeRequest.default)({
        url: `${endpoint}/api/snap-requests?payloadHash=${payloadHash}`,
        method: 'POST',
        json: true,
        formData
      }, {
        apiKey,
        apiSecret,
        retryCount: 5
      });
    };
    const promises = [];
    const requestIds = [];
    if (staticPackage) {
      for (let i = 0; i < this.chunks; i += 1) {
        // We allow one `await` inside the loop here to avoid POSTing all payloads
        // to the server at the same time (thus reducing load a little).
        // eslint-disable-next-line no-await-in-loop
        const {
          requestId
        } = await boundMakeRequest({
          chunk: {
            index: i,
            total: this.chunks
          }
        });
        if (asyncResults) {
          requestIds.push(requestId);
        } else {
          promises.push(waitFor({
            requestId,
            endpoint,
            apiKey,
            apiSecret
          }));
        }
      }
    } else if (pages) {
      for (const pageSlice of getPageSlices(pages, this.chunks)) {
        // We allow one `await` inside the loop here to avoid POSTing all payloads
        // to the server at the same time (thus reducing load a little).
        // eslint-disable-next-line no-await-in-loop
        const {
          requestId
        } = await boundMakeRequest({
          pageSlice
        });
        if (asyncResults) {
          requestIds.push(requestId);
        } else {
          promises.push(waitFor({
            requestId,
            endpoint,
            apiKey,
            apiSecret
          }));
        }
      }
    } else {
      const snapsPerChunk = Math.ceil(snapPayloads.length / this.chunks);
      for (let i = 0; i < this.chunks; i += 1) {
        const slice = snapPayloads.slice(i * snapsPerChunk, i * snapsPerChunk + snapsPerChunk);
        // We allow one `await` inside the loop here to avoid POSTing all payloads
        // to the server at the same time (thus reducing load a little).
        // eslint-disable-next-line no-await-in-loop
        const {
          requestId
        } = await boundMakeRequest({
          slice
        });
        if (asyncResults) {
          requestIds.push(requestId);
        } else {
          promises.push(waitFor({
            requestId,
            endpoint,
            apiKey,
            apiSecret
          }));
        }
      }
    }
    if (asyncResults) {
      return requestIds;
    }
    const result = [];
    (await Promise.all(promises)).forEach(list => {
      result.push(...list);
    });
    return result;
  }
}
exports.default = RemoteBrowserTarget;