"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = compareReports;
var _compareSnapshots = _interopRequireDefault(require("../compareSnapshots"));
var _makeRequest = _interopRequireDefault(require("../makeRequest"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
function ignore({
  before,
  after,
  apiKey,
  apiSecret,
  endpoint
}) {
  return (0, _makeRequest.default)({
    url: `${endpoint}/api/ignored-diffs`,
    method: 'POST',
    json: true,
    body: {
      snapshot1Id: before.id,
      snapshot2Id: after.id
    }
  }, {
    apiKey,
    apiSecret
  });
}
async function compareReports(sha1, sha2, {
  apiKey,
  apiSecret,
  endpoint,
  project,
  compareThreshold,
  afterSyncComparison
}, {
  link,
  message,
  author,
  dryRun,
  isAsync,
  notify,
  fallbackShas: rawFallbackShas
}, log = console.log, retryCount = 5) {
  const fallbackShas = rawFallbackShas ? rawFallbackShas.split(/[,\s]+/) : undefined;
  const makeCompareCall = skipStatusPost => (0, _makeRequest.default)({
    url: `${endpoint}/api/reports/${sha1}/compare/${sha2}`,
    method: 'POST',
    json: true,
    body: {
      link,
      message,
      author,
      project,
      skipStatusPost,
      isAsync,
      notify,
      fallbackShas
    }
  }, {
    apiKey,
    apiSecret,
    retryCount
  });
  const firstCompareResult = await makeCompareCall(typeof compareThreshold === 'number' || dryRun);
  if (typeof compareThreshold !== 'number' || isAsync) {
    if (!isAsync && afterSyncComparison && typeof afterSyncComparison === 'function') {
      await afterSyncComparison(firstCompareResult);
    }
    // We're not using a threshold -- return results right away
    return firstCompareResult;
  }
  const resolved = [];
  log(`Found ${firstCompareResult.diffs.length} diffs to deep-compare using threshold ${compareThreshold}`);
  if (dryRun) {
    log('Running in --dry-run mode -- no destructive commands will be issued');
  }
  const diffsClone = firstCompareResult.diffs.slice(0);
  let batch;
  // eslint-disable-next-line no-cond-assign
  while ((batch = diffsClone.splice(0, 10)).length > 0) {
    // Batch logs to help avoid running out of file descriptors
    const linesToLog = [];

    // eslint-disable-next-line no-await-in-loop
    await Promise.all(batch.map(async ([before, after]) => {
      const firstDiffDistance = await (0, _compareSnapshots.default)({
        before,
        after,
        endpoint,
        compareThreshold,
        retries: retryCount,
        apiKey,
        apiSecret
      });
      if (!firstDiffDistance) {
        linesToLog.push(`✓ ${after.component} - ${after.variant} - ${after.target} - diff below threshold, auto-ignoring`);
        if (!dryRun) {
          await ignore({
            before,
            after,
            apiKey,
            apiSecret,
            endpoint
          });
        }
        resolved.push([before, after]);
      }
    }));
    if (linesToLog.length) {
      log(linesToLog.join('\n'));
    }
  }
  const totalDiffsCount = firstCompareResult.diffs.length;
  const autoIgnoredDiffsCount = resolved.length;
  log(`${autoIgnoredDiffsCount} out of ${totalDiffsCount} were below threshold and auto-ignored`);

  // Make second compare call to finalize the deep compare. The second call will
  // cause a status to be posted to the PR (if applicable). Any ignored diffs
  // from the first call will be excluded from the result.
  const secondCompareResult = await makeCompareCall(false);
  log(secondCompareResult.summary);
  if (!isAsync && afterSyncComparison && typeof afterSyncComparison === 'function') {
    await afterSyncComparison(secondCompareResult);
  }
  return _objectSpread({
    resolved
  }, secondCompareResult);
}