"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = postGithubComment;
var _nodeFetch = _interopRequireDefault(require("node-fetch"));
var _Logger = _interopRequireDefault(require("./Logger"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const REPO_URL_MATCHER = /https?:\/\/[^/]+\/([^/]+)\/([^/]+)\/pull\/([0-9]+)/;
// https://github.com/lightstep/lightstep/pull/6555

const {
  HAPPO_GITHUB_USER_CREDENTIALS
} = process.env;
async function postGithubComment({
  statusImageUrl,
  compareUrl,
  link,
  githubApiUrl,
  userCredentials = HAPPO_GITHUB_USER_CREDENTIALS
}) {
  const matches = link.match(REPO_URL_MATCHER);
  if (!matches) {
    new _Logger.default().info(`URL does not look like a github PR URL: ${link}. Skipping github comment posting...`);
    return false;
  }
  const [user, pass] = userCredentials.split(':');
  const owner = matches[1];
  const repo = matches[2];
  const prNumber = matches[3];
  const normalizedGithubApiUrl = githubApiUrl.replace(/\/$/, '');
  const auth = Buffer.from(`${user}:${pass}`).toString('base64');
  const body = `[![Happo status](${statusImageUrl})](${compareUrl})`;
  const res = await (0, _nodeFetch.default)(`${normalizedGithubApiUrl}/repos/${owner}/${repo}/issues/${prNumber}/comments`, {
    method: 'POST',
    json: true,
    headers: {
      'User-Agent': 'happo.io client',
      Authorization: `Basic ${auth}`
    },
    body: JSON.stringify({
      body
    })
  });
  if (!res.ok) {
    throw new Error(`Failed to post github comment: ${res.status} ${await res.text()}`);
  }
  return true;
}