"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = remoteRunner;
var _crypto = _interopRequireDefault(require("crypto"));
var _fs = _interopRequireDefault(require("fs"));
var _os = _interopRequireDefault(require("os"));
var _path = _interopRequireDefault(require("path"));
var _archiver = _interopRequireDefault(require("archiver"));
var _createStaticPackage = require("./createStaticPackage");
var _Logger = _interopRequireWildcard(require("./Logger"));
var _constructReport = _interopRequireDefault(require("./constructReport"));
var _createHash = _interopRequireDefault(require("./createHash"));
var _ensureTarget = _interopRequireDefault(require("./ensureTarget"));
var _loadCSSFile = _interopRequireDefault(require("./loadCSSFile"));
var _uploadAssets = _interopRequireDefault(require("./uploadAssets"));
var _validateArchive = _interopRequireDefault(require("./validateArchive"));
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function staticDirToZipFile(dir) {
  return new Promise((resolve, reject) => {
    const archive = new _archiver.default('zip');
    const rnd = _crypto.default.randomBytes(4).toString('hex');
    const pathToZipFile = _path.default.join(_os.default.tmpdir(), `happo-static-${rnd}.zip`);
    const output = _fs.default.createWriteStream(pathToZipFile);
    const entries = [];
    archive.on('entry', entry => {
      entries.push(entry);
    });
    output.on('finish', async () => {
      (0, _validateArchive.default)(archive.pointer(), entries);
      resolve(pathToZipFile);
    });
    archive.pipe(output);
    archive.directory(dir, false, {
      date: _createStaticPackage.FILE_CREATION_DATE
    });
    archive.on('error', reject);
    archive.finalize();
  });
}
async function resolvePackageData(staticPackage) {
  if (typeof staticPackage === 'string') {
    // legacy plugins
    const buffer = Buffer.from(staticPackage, 'base64');
    return {
      value: buffer,
      hash: (0, _createHash.default)(buffer)
    };
  }
  if (!staticPackage.path) {
    throw new Error('Expected `staticPackage` to be an object with the following structure: `{ path: "path/to/folder" }`');
  }
  const file = await staticDirToZipFile(staticPackage.path);
  const readStream = _fs.default.createReadStream(file);
  const hash = await new Promise(resolve => {
    const hashCreator = _crypto.default.createHash('md5');
    readStream.pipe(hashCreator);
    hashCreator.setEncoding('hex');
    readStream.on('end', () => {
      hashCreator.end();
      resolve(hashCreator.read());
    });
  });
  readStream.destroy();
  return {
    value: _fs.default.createReadStream(file),
    hash
  };
}
async function remoteRunner({
  apiKey,
  apiSecret,
  endpoint,
  targets,
  plugins,
  stylesheets,
  project
}, {
  generateStaticPackage
}, {
  isAsync
}) {
  const logger = new _Logger.default();
  try {
    logger.info(`${(0, _Logger.logTag)(project)}Generating static package...`);
    const staticPackage = await generateStaticPackage();
    const {
      value,
      hash
    } = await resolvePackageData(staticPackage);
    const staticPackagePath = await (0, _uploadAssets.default)(value, {
      hash,
      endpoint,
      apiSecret,
      apiKey,
      logger,
      project
    });
    const targetNames = Object.keys(targets);
    const tl = targetNames.length;
    const cssBlocks = await Promise.all(stylesheets.map(_loadCSSFile.default));
    plugins.forEach(({
      css
    }) => cssBlocks.push(css || ''));
    logger.info(`${(0, _Logger.logTag)(project)}Generating screenshots in ${tl} target${tl > 1 ? 's' : ''}...`);
    const outerStartTime = Date.now();
    const results = await Promise.all(targetNames.map(async name => {
      const startTime = Date.now();
      const result = await (0, _ensureTarget.default)(targets[name]).execute({
        targetName: name,
        asyncResults: isAsync,
        staticPackage: staticPackagePath,
        apiKey,
        apiSecret,
        endpoint,
        globalCSS: cssBlocks.join('').replace(/\n/g, '')
      });
      logger.start(`  - ${(0, _Logger.logTag)(project)}${name}`, {
        startTime
      });
      logger.success();
      return {
        name,
        result
      };
    }));
    logger.start(undefined, {
      startTime: outerStartTime
    });
    logger.success();
    if (isAsync) {
      return results;
    }
    return (0, _constructReport.default)(results);
  } catch (e) {
    logger.fail();
    throw e;
  }
}