"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = uploadAssets;
var _Logger = require("./Logger");
var _makeRequest = _interopRequireDefault(require("./makeRequest"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
async function uploadAssets(streamOrBuffer, {
  hash,
  endpoint,
  apiKey,
  apiSecret,
  logger,
  project
}) {
  try {
    // Check if the assets already exist. If so, we don't have to upload them.
    const assetsDataRes = await (0, _makeRequest.default)({
      url: `${endpoint}/api/snap-requests/assets-data/${hash}`,
      method: 'GET',
      json: true
    }, {
      apiKey,
      apiSecret
    });
    logger.info(`${(0, _Logger.logTag)(project)}Reusing existing assets at ${assetsDataRes.path} (previously uploaded on ${assetsDataRes.uploadedAt})`);
    return assetsDataRes.path;
  } catch (e) {
    if (e.statusCode !== 404) {
      logger.warn(`${(0, _Logger.logTag)(project)}Assuming assets don't exist since we got error response: ${e.statusCode} - ${e.message} - ${e.stack}`);
    }
  }
  const assetsRes = await (0, _makeRequest.default)({
    url: `${endpoint}/api/snap-requests/assets/${hash}`,
    method: 'POST',
    json: true,
    formData: {
      payload: {
        options: {
          filename: 'payload.zip',
          contentType: 'application/zip'
        },
        value: streamOrBuffer
      }
    }
  }, {
    apiKey,
    apiSecret,
    retryCount: 2
  });
  return assetsRes.path;
}